@@ -92,7 +92,7 @@ extern int parse_loss_vec(category_count_t i, char *buf);
 extern int parse_risk_spec(FILE *input);
 
 bool_t found_scores = 0;
-int cmd = 0;
+extern int cmd;
 int exit_code = 0; /* default */
 
 /***********************************************************
@@ -210,8 +210,9 @@ int parse_dbacl_scores(char *buf) {
   char *tok;
   category_count_t index, counter;
   
+
   /* first token is scores */
-  tok = strtok(buf, " \n");
+  tok = strtok(buf, "# \n");
 
   counter = 0;
   /* next gobble up tokens four at a time */
@@ -521,7 +522,7 @@ int read_riskspec(char *filename) {
  * encoding doesn't include NUL bytes inside characters    *
  ***********************************************************/
 
-#define MAGIC "scores "
+#define MAGIC "# scores "
 #define MULTIBYTE_EPSILON 10 /* enough for a multibyte char and a null char */
 
 
@@ -542,7 +543,7 @@ void b_process_file(FILE *input,
     if( line_fun ) { (*line_fun)(); }
 
 
-    if( (textbuf[0] == 's') && 
+    if( (textbuf[0] == '#') && 
 	(strncmp(MAGIC, textbuf, 7) == 0) ) {
       if( !parse_dbacl_scores(textbuf) ) {
 	errormsg(E_FATAL,"scores don't match risk specification\n");
