@@ -96,6 +96,7 @@ char *print_model_options(options_t opt, char *buf) {
   strcat(buf, MOPTION(M_OPTION_SHOW_ALT, opt));
   strcat(buf, MOPTION(M_OPTION_HTML, opt));
   strcat(buf, MOPTION(M_OPTION_XHEADERS, opt));
+  strcat(buf, MOPTION(M_OPTION_THEADERS, opt));
   strcat(buf, MOPTION(M_OPTION_SHOW_SCRIPT, opt));
   strcat(buf, MOPTION(M_OPTION_SHOW_STYLE, opt));
   strcat(buf, MOPTION(M_OPTION_SHOW_HTML_COMMENTS, opt));
@@ -128,6 +129,7 @@ char *print_user_options(options_t opt, char *buf) {
   strcat(buf, MOPTION(U_OPTION_JAYNES, opt));
   strcat(buf, MOPTION(U_OPTION_MMAP, opt));
   strcat(buf, MOPTION(U_OPTION_CONFIDENCE, opt));
+  strcat(buf, MOPTION(U_OPTION_VAR, opt));
   return buf;
 }
 
@@ -142,6 +144,7 @@ error_code_t sanitize_model_options(options_t *mopt, category_t *cat) {
     (1<<M_OPTION_SHOW_STYLE)|
     (1<<M_OPTION_HEADERS)|
     (1<<M_OPTION_XHEADERS)|
+    (1<<M_OPTION_THEADERS)|
     (1<<M_OPTION_SHOW_ALT)|
     (1<<M_OPTION_SHOW_FORMS)|
     (1<<M_OPTION_NOHEADERS)|
@@ -405,6 +408,7 @@ void init_category(category_t *cat) {
     cat->model_unique_token_count = 0;
     cat->score = 0.0;
     cat->score_div = 0.0;
+    cat->score_s2 = 0.0;
     cat->score_shannon = 0.0;
     cat->shannon = 0.0;
     cat->alpha = 0.0;
@@ -663,6 +667,11 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
 	  break;
 	}
 
+	if( tt.order == 1 ) {
+	  /* sample variance */
+	  cat[i].score_s2 += ((oldscore - cat[i].score) * (oldscore - cat[i].score));
+	}
+
       }
 
       if( u_options & (1<<U_OPTION_DUMP) ) {
@@ -701,13 +710,13 @@ void score_word(char *tok, token_type_t tt, regex_count_t re) {
  * -2\sum_i=1^n \log X_i ~ chi^2(2n), if all X_i are IID uniform.
  *
  */
-int gamma_pvalue(category_t *cat, double obs) {
+confidence_t gamma_pvalue(category_t *cat, double obs) {
   double m;
   if( (obs <= 0.0) || (cat->alpha <= 0.0) || (cat->beta <= 0.0) ) {
-    return 1010; /* bogus value */
+    return 0; /* bogus value */
   } else if( obs > 120.0 ) {
     /* large values can produce underflow in gamma_tail() */
-    return 1010; /* bogus value */
+    return 0; /* bogus value */
   }
 
   m = gamma_tail(cat->alpha, cat->beta, obs);
@@ -715,7 +724,7 @@ int gamma_pvalue(category_t *cat, double obs) {
   /* uncomment below to get one sided confidence */
   /*   return (int)(1000.0 * m); */
   /* result is per-thousand */
-  return (int)(2000.0 * ((m > 0.5) ? 1.0 - m : m));
+  return (confidence_t)(2000.0 * ((m > 0.5) ? 1.0 - m : m));
 }
 
 /***********************************************************
