@@ -101,7 +101,7 @@ extern char *progname;
 extern char *inputfile;
 extern long inputline;
 
-int cmd = 0;
+extern int cmd;
 int exit_code = 0; /* default */
 
 int overflow_warning = 0;
@@ -181,11 +181,15 @@ void line_score_categories(char *textbuf) {
   /* find MAP */
   cmax = cat[0].score; 
   map = 0;
-  for(i = 1; i < cat_count; i++) {
+  for(i = 0; i < cat_count; i++) {
     if(cmax < cat[i].score) {
       cmax = cat[i].score;
       map = i;
     }
+    /* finish sample variance calculation */
+    cat[i].score_s2 = (cat[i].complexity * cat[i].score_s2 -
+		       cat[i].score * cat[i].score)/
+      ((score_t)cat[i].complexity * ((score_t)cat[i].complexity - 1));
   }
 
   if( !(u_options & (1<<U_OPTION_DUMP)) ) {
@@ -237,6 +241,7 @@ void line_score_categories(char *textbuf) {
   /* clean up for next line */
   for(i = 0; i < cat_count; i++) {
     cat[i].score = 0.0;
+    cat[i].score_s2 = 0.0;
     cat[i].score_div = 0.0;
     cat[i].score_shannon = 0.0;
     cat[i].complexity = 0;
@@ -254,11 +259,15 @@ void score_categories() {
   /* find MAP */
   cmax = cat[0].score; 
   exit_code = 0;
-  for(i = 1; i < cat_count; i++) {
+  for(i = 0; i < cat_count; i++) {
     if(cmax < cat[i].score) {
       cmax = cat[i].score;
       exit_code = i;
     }
+    /* finish sample variance calculation */
+    cat[i].score_s2 = (cat[i].complexity * cat[i].score_s2 -
+		       cat[i].score * cat[i].score)/
+      ((score_t)cat[i].complexity * ((score_t)cat[i].complexity - 1));
   }
 
   /* finish computing entropies */
@@ -305,19 +314,28 @@ void score_categories() {
     /* display logarithmic score */
     for(i = 0; i < cat_count; i++) {
       if( u_options & (1<<U_OPTION_VERBOSE) ) {
-	fprintf(stdout, "%s %5.2" FMT_printf_score_t " * %-d ", 
-		cat[i].filename, 
-		-CVT_BITS(cat[i].score/cat[i].complexity),
-		cat[i].complexity);
+	if( u_options & (1<<U_OPTION_VAR) ) {
+	  fprintf(stdout, "%s ( %5.2" FMT_printf_score_t 
+		  " # %5.2" FMT_printf_score_t " )* %-d ", 
+		  cat[i].filename, 
+		  -CVT_BITS(cat[i].score/cat[i].complexity),
+		  CVT_BITS(sqrt(cat[i].score_s2/cat[i].complexity)),
+		  cat[i].complexity);
+	} else {
+	  fprintf(stdout, "%s %5.2" FMT_printf_score_t " * %-d ", 
+		  cat[i].filename, 
+		  -CVT_BITS(cat[i].score/cat[i].complexity),
+		  cat[i].complexity);
+	}
+	if( u_options & (1<<U_OPTION_CONFIDENCE) ) {
+	  fprintf(stdout, "@ %5.1f%% ", 
+		  (float)gamma_pvalue(&cat[i], cat[i].score_div)/10);
+	}
       } else {
 	fprintf(stdout, "%s %5.2" FMT_printf_score_t " ", 
 		cat[i].filename,
 		-CVT_BITS(cat[i].score));
       }
-      if( u_options & (1<<U_OPTION_CONFIDENCE) ) {
-	fprintf(stdout, "@ %5.1f%% ", 
-		(float)gamma_pvalue(&cat[i], cat[i].score_div));
-      }
     }
     fprintf(stdout, "\n");
 
@@ -340,16 +358,23 @@ void score_categories() {
   } else {
 
     if( u_options & (1<<U_OPTION_VERBOSE) ) {
-      if( u_options & (1<<U_OPTION_CONFIDENCE) ) {
+      if( (u_options & (1<<U_OPTION_CONFIDENCE)) ||
+	  (u_options & (1<<U_OPTION_VAR)) ) {
 	for(i = 0; i < cat_count; i++) {
 	  fprintf(stdout, "%s ( %5.2" FMT_printf_score_t " + "
-		  "%-5.2" FMT_printf_score_t " )* %-5d @ %5.1f%% ", 
+		  "%-5.2" FMT_printf_score_t,
 		  cat[i].filename, 
 		  CVT_BITS(cat[i].score_div),
-		  CVT_BITS(cat[i].score_shannon),
-		  cat[i].complexity,
-		  (float)gamma_pvalue(&cat[i], cat[i].score_div)/10);
-
+		  CVT_BITS(cat[i].score_shannon));
+	  if( u_options & (1<<U_OPTION_VAR) ) {
+	    fprintf(stdout, " # %5.2" FMT_printf_score_t,
+		    CVT_BITS(sqrt(cat[i].score_s2/cat[i].complexity)));
+	  }
+	  fprintf(stdout, " )* %-5d", cat[i].complexity);
+	  if( u_options & (1<<U_OPTION_CONFIDENCE) ) {
+	    fprintf(stdout, " @ %5.1f%% ", 
+		    (float)gamma_pvalue(&cat[i], cat[i].score_div)/10);
+	  }
 	}
 	fprintf(stdout, "\n");
       } else {
@@ -358,6 +383,37 @@ void score_categories() {
 	}
 	fprintf(stdout, "%s\n", cat[exit_code].filename); 
       }
+    } else {
+      if( u_options & (1<<U_OPTION_VAR) ) {
+	if( u_options & (1<<U_OPTION_APPEND) ) {
+	  fprintf(stdout, "\n# category ");
+	}
+
+	cmax = 1.0;
+	for(i = 0; i < cat_count; i++) {
+	  if( i != exit_code ) {
+	    /* c is a standard normal variable */
+	    c = (-cat[i].score/cat[i].complexity - 
+		 -cat[exit_code].score/cat[exit_code].complexity) /
+	      sqrt(cat[i].score_s2/cat[i].complexity + 
+		   cat[exit_code].score_s2/cat[exit_code].complexity);
+	    /* c will always be positive, but let's be safe */
+	    if( isnan(c) ) {
+	      c = 0.0;
+	    } else if( c > 5.0 ) {
+	      c = 1.0;
+	    } else if( c < -5.0 ) {
+	      c = 0.0;
+	    } else {
+	      /* normal_cdf(c) has a range of 0.5 - 1.0, we convert to a full range */
+	      c = 2 * normal_cdf(c) - 1.0;
+	    }
+	    cmax = (cmax > c) ? c : cmax; 
+	  }
+	}
+	fprintf(stdout, "%s # %d%%\n", 
+		cat[exit_code].filename, (int)ceil(cmax * 100.0)); 
+      }
     }
 
   }
@@ -406,7 +462,7 @@ FILE *mytmpfile(const char *tmplate, char **tmpname) {
     for(i = 0; i < 100; i++) {
       snprintf(*tmpname + l, 3, ".%d", i);
       /* this may have problems on NFS systems? */
-      fd = open(*tmpname, O_CREAT|O_EXCL|O_WRONLY, 0440);
+      fd = ATOMIC_CREATE(*tmpname);
       if( fd != -1 ) {
 	result = fdopen(fd, "wb");
 	/* no need to close fd */
@@ -2776,6 +2832,9 @@ int set_option(int op, char *optarg) {
     } else if( !strcasecmp(optarg, "email:xheaders") ) {
       m_options |= (1<<M_OPTION_XHEADERS);
       m_options |= (1<<M_OPTION_MBOX_FORMAT);
+    } else if( !strcasecmp(optarg, "email:theaders") ) {
+      m_options |= (1<<M_OPTION_THEADERS);
+      m_options |= (1<<M_OPTION_MBOX_FORMAT);
     } else if( !strcasecmp(optarg, "email") ) {
       m_options |= (1<<M_OPTION_MBOX_FORMAT);
     } else if( !strcasecmp(optarg, "html:links") ) {
@@ -2807,6 +2866,9 @@ int set_option(int op, char *optarg) {
     }
     c++;
     break;
+  case 'U':
+    u_options |= (1<<U_OPTION_VAR);
+    break;
   case 'V':
     fprintf(stdout, "dbacl version %s\n", VERSION);
     fprintf(stdout, COPYBLURB, "dbacl");
@@ -3191,14 +3253,20 @@ int main(int argc, char **argv) {
 
   category_count_t c;
   FILE *input;
-  struct stat statinfo;
   signed char op;
+  struct stat statinfo;
 
   void (*preprocess_fun)(void) = NULL;
   void (*word_fun)(char *, token_type_t, regex_count_t) = NULL;
   char *(*pre_line_fun)(char *) = NULL;
   void (*post_line_fun)(char *) = NULL;
   void (*postprocess_fun)(void) = NULL;
+  int (*line_filter)(MBOX_State *, char *) = NULL;
+  void (*character_filter)(XML_State *, char *) = NULL; 
+#if defined HAVE_MBRTOWC
+  int (*w_line_filter)(MBOX_State *, wchar_t *) = NULL;
+  void (*w_character_filter)(XML_State *, wchar_t *) = NULL; 
+#endif
 
   progname = "dbacl";
   inputfile = "stdin";
@@ -3227,7 +3295,7 @@ int main(int argc, char **argv) {
 
   /* parse the options */
   while( (op = getopt(argc, argv, 
-		      "0Aac:Dde:f:G:g:H:h:ijL:l:mMNno:q:RrT:Vvw:x:X")) > -1 ) {
+		      "0Aac:Dde:f:G:g:H:h:ijL:l:mMNno:q:RrT:UVvw:x:X")) > -1 ) {
     set_option(op, optarg);
   }
 
@@ -3283,7 +3351,7 @@ int main(int argc, char **argv) {
       fprintf(stdout, "\n# format: ");
 
       if( u_options & (1<<U_OPTION_POSTERIOR) ) {
-	fprintf(stdout, "score_delta");
+	fprintf(stdout, "score_delta\n");
       } else if( u_options & (1<<U_OPTION_SCORES) ) {
 	fprintf(stdout, "avg_score * complexity\n");
       } else {
@@ -3314,6 +3382,27 @@ int main(int argc, char **argv) {
     pre_line_fun = handle_indents_and_appends;
   }
 
+  if( m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
+    line_filter = email_line_filter;
+  }
+
+  if( (m_options & (1<<M_OPTION_XML)) ||
+      (m_options & (1<<M_OPTION_HTML)) ) {
+    character_filter = xml_character_filter;
+  }
+
+#if defined HAVE_MBRTOWC
+  if( m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
+    w_line_filter = w_email_line_filter;
+  }
+
+  if( (m_options & (1<<M_OPTION_XML)) ||
+      (m_options & (1<<M_OPTION_HTML)) ) {
+    w_character_filter = w_xml_character_filter;
+  }
+#endif
+
+
   if( preprocess_fun ) { (*preprocess_fun)(); }
 
 
@@ -3326,28 +3415,6 @@ int main(int argc, char **argv) {
     input = fopen(argv[optind], "r");
     if( input ) {
       inputfile = argv[optind];
-      if( in_iobuf ) {
-	/* choose appropriate buffering mode */
-	if( fstat(fileno(input), &statinfo) == 0 ) {
-	  switch(statinfo.st_mode & S_IFMT) {
-	  case S_IFREG:
-	  case S_IFBLK:
-	    setvbuf(input, (char *)in_iobuf, _IOFBF, BUFFER_MAG * system_pagesize);
-	    break;
-	  case S_IFIFO:
-	  case S_IFCHR:
-	    setvbuf(input, (char *)NULL, 
-		    (u_options & (1<<U_OPTION_FILTER)) ? _IOLBF : _IOFBF, 
-		    BUFFER_MAG * system_pagesize);
-	    break;
-	  case S_IFDIR:
-	  default:
-	    /* nothing */
-	    break;
-	  }
-
-	}
-      }
 
       u_options |= (1<<U_OPTION_STDIN);
 
@@ -3358,33 +3425,40 @@ int main(int argc, char **argv) {
 
       /* set some initial options */
       reset_xml_character_filter(&xml, xmlRESET);
-
+      
       if( m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
 	reset_mbox_line_filter(&mbox);
       }
 
-      if( !(m_options & (1<<M_OPTION_I18N)) ) {
-	process_file(input, 
-		     ((m_options & (1<<M_OPTION_MBOX_FORMAT)) ? 
-		      email_line_filter : NULL), 
-		     (((m_options & (1<<M_OPTION_XML)) || 
-		       (m_options & (1<<M_OPTION_HTML))) ? 
-		      xml_character_filter : NULL), 
-		     word_fun, pre_line_fun, post_line_fun);
-      } else {
+      if( fstat(fileno(input), &statinfo) == 0 ) {
+	switch(statinfo.st_mode & S_IFMT) {
+	case S_IFDIR:
+	  if( !(m_options & (1<<M_OPTION_I18N)) ) {
+	    process_directory(inputfile, line_filter, character_filter,
+			      word_fun, pre_line_fun, post_line_fun);
+	  } else {
 #if defined HAVE_MBRTOWC
-	w_process_file(input, 
-		       ((m_options & (1<<M_OPTION_MBOX_FORMAT)) ? 
-			w_email_line_filter : NULL), 
-		       (((m_options & (1<<M_OPTION_XML)) || 
-			 (m_options & (1<<M_OPTION_HTML))) ? 
-			w_xml_character_filter : NULL), 
-		       word_fun, pre_line_fun, post_line_fun);
+	    w_process_directory(inputfile, w_line_filter, w_character_filter,
+				word_fun, pre_line_fun, post_line_fun);
 #else
-	errormsg(E_ERROR, "international support not available (recompile).\n");
+	    errormsg(E_ERROR, "international support not available (recompile).\n");
 #endif
+	  }
+	  break;
+	default:
+	  if( !(m_options & (1<<M_OPTION_I18N)) ) {
+	    process_file(input, line_filter, character_filter,
+			 word_fun, pre_line_fun, post_line_fun);
+	  } else {
+#if defined HAVE_MBRTOWC
+	    w_process_file(input, w_line_filter, w_character_filter,
+			   word_fun, pre_line_fun, post_line_fun);
+#else
+	    errormsg(E_ERROR, "international support not available (recompile).\n");
+#endif
+	  }
+	}
       }
-
       fclose(input);
 
     } else { /* unrecognized file name */
@@ -3404,29 +3478,6 @@ int main(int argc, char **argv) {
       fprintf(stdout, "taking input from stdin\n");
     }
 
-    if( in_iobuf ) {
-      /* choose appropriate buffering mode */
-      if( fstat(fileno(stdin), &statinfo) == 0 ) {
-	switch(statinfo.st_mode & S_IFMT) {
-	case S_IFREG:
-	case S_IFBLK:
-	  setvbuf(stdin, (char *)in_iobuf, _IOFBF, BUFFER_MAG * system_pagesize);
-	  break;
-	case S_IFIFO:
-	case S_IFCHR:
-	  setvbuf(stdin, (char *)NULL, 
-		  (u_options & (1<<U_OPTION_FILTER)) ? _IOLBF : _IOFBF, 
-		  BUFFER_MAG * system_pagesize);
-	  break;
-	case S_IFDIR:
-	default:
-	  /* nothing */
-	  break;
-	}
-
-      }
-    }
-
     /* set some initial options */
     reset_xml_character_filter(&xml, xmlRESET);
 
@@ -3435,21 +3486,11 @@ int main(int argc, char **argv) {
     }
 
     if( !(m_options & (1<<M_OPTION_I18N)) ) {
-      process_file(stdin, 
-		   ((m_options & (1<<M_OPTION_MBOX_FORMAT)) ? 
-		    email_line_filter : NULL), 
-		   (((m_options & (1<<M_OPTION_XML)) || 
-		     (m_options & (1<<M_OPTION_HTML))) ? 
-		    xml_character_filter : NULL), 
+      process_file(stdin, line_filter, character_filter,
 		   word_fun, pre_line_fun, post_line_fun);
     } else {
 #if defined HAVE_MBRTOWC
-      w_process_file(stdin, 
-		     ((m_options & (1<<M_OPTION_MBOX_FORMAT)) ? 
-		      w_email_line_filter : NULL), 
-		     (((m_options & (1<<M_OPTION_XML)) || 
-		       (m_options & (1<<M_OPTION_HTML))) ? 
-		      w_xml_character_filter : NULL), 
+      w_process_file(stdin, w_line_filter, w_character_filter,
 		     word_fun, pre_line_fun, post_line_fun);
 #else
       errormsg(E_ERROR, "international support not available (recompile).\n");
