@@ -43,6 +43,7 @@
  * trying to read the category while it is being written.
  */
 #define ATOMIC_CATSAVE
+#define ATOMIC_CREATE(x) open(x, O_CREAT|O_EXCL|O_WRONLY, 0440)
 
 /* we define several memory models, which differ basically 
    in the number of bytes used for the hash tables. Adjust to taste */
@@ -61,8 +62,7 @@
    (if you often get digitization errors, undef the appropriate macro) */
 
 /* digram digitization: avg loss of precision = 0.01 * token size */
-/* don't use this if you expect big datasets */
-#undef DIGITIZE_DIGRAMS
+#define DIGITIZE_DIGRAMS
 /* lambda digitization: avg loss of precision = 0.01 */
 #define DIGITIZE_LAMBDA
 /* learner.hash digitization: avg loss of precision = 0.01 */
@@ -154,6 +154,7 @@ typedef unsigned int hash_percentage_t;
 typedef u_int8_t category_count_t;
 typedef u_int8_t regex_count_t;
 typedef u_int32_t document_count_t;
+typedef u_int16_t confidence_t;
 
 typedef float weight_t;
 typedef long double score_t;
@@ -207,6 +208,7 @@ typedef unsigned int hash_percentage_t;
 typedef u_int8_t category_count_t;
 typedef u_int8_t regex_count_t;
 typedef u_int32_t document_count_t;
+typedef u_int16_t confidence_t;
 
 typedef float weight_t;
 typedef double score_t;
@@ -259,6 +261,7 @@ typedef unsigned int hash_percentage_t;
 typedef u_int8_t category_count_t;
 typedef u_int8_t regex_count_t;
 typedef u_int16_t document_count_t;
+typedef u_int16_t confidence_t;
 
 typedef float weight_t;
 typedef double score_t;
@@ -313,6 +316,7 @@ typedef unsigned int hash_percentage_t;
 typedef u_int8_t category_count_t;
 typedef u_int8_t regex_count_t;
 typedef u_int8_t document_count_t;
+typedef u_int16_t confidence_t;
 
 typedef float weight_t;
 typedef double score_t;
@@ -445,6 +449,7 @@ typedef u_int16_t digitized_weight_t;
 #define U_OPTION_JAYNES                 20
 #define U_OPTION_MMAP                   21
 #define U_OPTION_CONFIDENCE             22
+#define U_OPTION_VAR                    23
 
 /* model options */
 #define M_OPTION_REFMODEL               1
@@ -476,6 +481,7 @@ typedef u_int16_t digitized_weight_t;
 #define M_OPTION_SHOW_FORMS             28
 #define M_OPTION_NOHEADERS              29
 #define M_OPTION_NGRAM_STRADDLE_NL      30
+#define M_OPTION_THEADERS               31
 
 /* category options */
 #define C_OPTION_MMAPPED_HASH            1
@@ -681,6 +687,7 @@ typedef struct {
   score_t divergence;
   score_t score;
   score_t score_div;
+  score_t score_s2;
   score_t score_shannon;
   score_t shannon;
   score_t alpha;
@@ -810,11 +817,11 @@ typedef struct {
   MIME_Content_Encoding encoding;
 } MIME_Struct;
 
-typedef enum { htSTANDARD, htEXTENDED, htMIME, htCONT, htUNDEF } HEADER_Type;
+typedef enum { htSTANDARD, htEXTENDED, htTRACE, htMIME, htCONT, htUNDEF } HEADER_Type;
 
 typedef enum { msUNDEF=1, msHEADER, msBODY, msATTACH} Mstate;
 typedef enum { msuUNDEF=1, msuTRACK, msuMIME, msuARMOR, msuOTHER } Msubstate;
-typedef enum { mhsUNDEF=1, mhsSUBJECT, mhsFROM, mhsTO, mhsMIME, mhsXHEADER} Mhstate;
+typedef enum { mhsUNDEF=1, mhsSUBJECT, mhsFROM, mhsTO, mhsMIME, mhsXHEADER, mhsTRACE} Mhstate;
 typedef enum { maUNDEF=1, maENABLED} Marmor;
 typedef enum { psPLAIN, psUUENCODE } Mplainstate;
 
@@ -846,7 +853,10 @@ typedef struct {
   bool_t skip_until_boundary;
   bool_t corruption_check;
   bool_t skip_header;
-  bool_t strip_header_comment;
+  char strip_header_char;
+#if defined HAVE_MBRTOWC
+  wchar_t w_strip_header_char;
+#endif
   Mplainstate plainstate;
   struct {
     int size[MAX_BOUNDARIES];
@@ -932,7 +942,7 @@ extern "C"
   void reload_all_categories();
 
   void score_word(char *tok, token_type_t tt, regex_count_t re);
-  int gamma_pvalue(category_t *cat, double obs);
+  confidence_t gamma_pvalue(category_t *cat, double obs);
 
   /* file format handling in fh.c */
   void init_file_handling();
@@ -974,6 +984,12 @@ extern "C"
 		    void (*word_fun)(char *, token_type_t, regex_count_t), 
 		    char *(*pre_line_fun)(char *),
 		    void (*post_line_fun)(char *));
+  void process_directory(char *name, 
+			 int (*line_filter)(MBOX_State *, char *),
+			 void (*character_filter)(XML_State *, char *), 
+			 void (*word_fun)(char *, token_type_t, regex_count_t), 
+			 char *(*pre_line_fun)(char *),
+			 void (*post_line_fun)(char *));
 
   void init_mbox_line_filter(MBOX_State *mbox);
   void free_mbox_line_filter(MBOX_State *mbox);
@@ -1021,6 +1037,12 @@ extern "C"
 		      void (*word_fun)(char *, token_type_t, regex_count_t), 
 		      char *(*pre_line_fun)(char *),
 		      void (*post_line_fun)(char *));
+  void w_process_directory(char *name,
+			   int (*line_filter)(MBOX_State *, wchar_t *),
+			   void (*character_filter)(XML_State *, wchar_t *), 
+			   void (*word_fun)(char *, token_type_t, regex_count_t), 
+			   char *(*pre_line_fun)(char *),
+			   void (*post_line_fun)(char *));
 
 #endif
 
