@@ -29,6 +29,8 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <dirent.h>
+#include <sys/stat.h>
 
 #if defined HAVE_UNISTD_H
 #include <unistd.h> 
@@ -67,6 +69,7 @@ extern void *out_iobuf;
 
 extern int cmd;
 
+extern char *inputfile;
 extern long inputline;
 
 /***********************************************************
@@ -214,44 +217,13 @@ char *MMGETS(char *s, size_t size, MMFILE *stream) {
  ***********************************************************/
 
 void init_file_handling() {
-  /* preallocate primary text holding buffer */
-  textbuf_len = system_pagesize;
-  textbuf = (char *)malloc(textbuf_len);
-
+  init_buffers();
   init_mbox_line_filter(&mbox);
-
-  MADVISE(textbuf, sizeof(char) * textbuf_len, MADV_SEQUENTIAL);
-
-#if defined HAVE_POSIX_MEMALIGN
-  /* buffer must exist until after fclose() if used in setvbuf() */
-  if( 0 != posix_memalign(&in_iobuf, system_pagesize, 
-			  BUFFER_MAG * system_pagesize) ) {
-    in_iobuf = NULL; /* just to be sure */
-  }
-  /* buffer must exist until after fclose() if used in setvbuf() */
-  if( 0 != posix_memalign(&out_iobuf, system_pagesize, 
-			  BUFFER_MAG * system_pagesize) ) {
-    out_iobuf = NULL; /* just to be sure */
-  }
-#elif defined HAVE_MEMALIGN
-  /* memalign()ed memory can't be reclaimed by free() */
-  in_iobuf = (void *)memalign(system_pagesize, BUFFER_MAG * system_pagesize);
-  out_iobuf = (void *)memalign(system_pagesize, BUFFER_MAG * system_pagesize);
-#elif defined HAVE_VALLOC
-  /* valloc()ed memory can't be reclaimed by free() */
-  in_iobuf = (void *)valloc(BUFFER_MAG * system_pagesize);
-  out_iobuf = (void *)valloc(BUFFER_MAG * system_pagesize);
-#endif
 }
 
 void cleanup_file_handling() {
-  /* free some global resources */
-  free(textbuf);
   free_mbox_line_filter(&mbox);
-#if defined HAVE_POSIX_MEMALIGN
-  if( in_iobuf ) { free(in_iobuf); }
-  if( out_iobuf ) { free(out_iobuf); }
-#endif
+  cleanup_buffers();
 }
 
 /* Given a mime type, this selects an appropriate
@@ -340,7 +312,10 @@ void reset_mbox_line_filter(MBOX_State *mbox) {
   mbox->corruption_check = 0;
   mbox->skip_header = 0;
   mbox->skip_until_boundary = 0;
-  mbox->strip_header_comment = 0;
+  mbox->strip_header_char = '\0';
+#if defined HAVE_MBRTOWC
+  mbox->w_strip_header_char = L'\0';
+#endif
   mbox->plainstate = psPLAIN;
   memset(&mbox->boundary, 0, sizeof(mbox->boundary));
 
@@ -386,6 +361,9 @@ token_type_t get_token_type(token_order_t o) {
     switch(mbox.state) {
     case msHEADER:
       switch(mbox.hstate) {
+      case mhsTRACE:
+	tt.cls = AMIN + 9;
+	break;
       case mhsFROM:
 	tt.cls = AMIN + 8;
 	break;
@@ -437,6 +415,57 @@ bool_t finalize_textbuf(int *final) {
   return 0;
 }
 
+void process_directory(char *name,
+		       int (*line_filter)(MBOX_State *, char *),
+		       void (*character_filter)(XML_State *, char *), 
+		       void (*word_fun)(char *, token_type_t, regex_count_t), 
+		       char *(*pre_line_fun)(char *),
+		       void (*post_line_fun)(char *)) {
+  DIR *d;
+  struct dirent *sd;
+  FILE *input;
+  struct stat statinfo;
+  char fullp[_POSIX_PATH_MAX + 1];
+  char *fp;
+
+  d = opendir(name);
+  if( d ) {
+    /* directory returns relative file names, but we need full paths */
+    strcpy(fullp, name);
+    fp = fullp + strlen(name); 
+    *fp++ = '/';
+
+    for(sd = readdir(d); sd; sd = readdir(d)) {
+      strcpy(fp, sd->d_name);
+
+      if( stat(fullp, &statinfo) == 0 ) {
+	switch(statinfo.st_mode & S_IFMT) {
+	case S_IFREG:
+	  input = fopen(fullp, "r");
+	  if( input ) {
+	    inputfile = fullp;
+	    /* set some initial options */
+	    reset_xml_character_filter(&xml, xmlRESET);
+      
+	    if( m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
+	      reset_mbox_line_filter(&mbox);
+	    }
+	    process_file(input, line_filter, character_filter, 
+			 word_fun, pre_line_fun, post_line_fun);
+	    fclose(input);
+	  }
+	default:
+	  /* nothing */
+	  break;
+	}
+      }
+    }
+    closedir(d);
+  } else {
+    errormsg(E_WARNING, "could not open %s, skipping\n", name);
+  }
+}
+
 /* reads a text file as input and applies several filters. */
 void process_file(FILE *input, 
 		  int (*line_filter)(MBOX_State *, char *),
@@ -458,6 +487,9 @@ void process_file(FILE *input,
   q = tokbuf + 1;
   how_many = 0;
 
+
+  set_iobuf_mode(input);
+
   inputline = 0;
 
   /* now start processing */
@@ -526,6 +558,57 @@ void process_file(FILE *input,
  ***********************************************************/
 #if defined HAVE_MBRTOWC
 
+void w_process_directory(char *name,
+			 int (*w_line_filter)(MBOX_State *, wchar_t *),
+			 void (*w_character_filter)(XML_State *, wchar_t *), 
+			 void (*word_fun)(char *, token_type_t, regex_count_t), 
+			 char *(*pre_line_fun)(char *),
+			 void (*post_line_fun)(char *)) {
+  DIR *d;
+  struct dirent *sd;
+  FILE *input;
+  struct stat statinfo;
+  char fullp[_POSIX_PATH_MAX + 1];
+  char *fp;
+
+  d = opendir(name);
+  if( d ) {
+    /* directory returns relative file names, but we need full paths */
+    strcpy(fullp, name);
+    fp = fullp + strlen(name); 
+    *fp++ = '/';
+
+    for(sd = readdir(d); sd; sd = readdir(d)) {
+      strcpy(fp, sd->d_name);
+
+      if( stat(fullp, &statinfo) == 0 ) {
+	switch(statinfo.st_mode & S_IFMT) {
+	case S_IFREG:
+	  input = fopen(fullp, "r");
+	  if( input ) {
+	    inputfile = fullp;
+	    /* set some initial options */
+	    reset_xml_character_filter(&xml, xmlRESET);
+      
+	    if( m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
+	      reset_mbox_line_filter(&mbox);
+	    }
+	    w_process_file(input, w_line_filter, w_character_filter, 
+			   word_fun, pre_line_fun, post_line_fun);
+	    fclose(input);
+	  }
+	default:
+	  /* nothing */
+	  break;
+	}
+      }
+    }
+    closedir(d);
+  } else {
+    errormsg(E_WARNING, "could not open %s, skipping\n", name);
+  }
+}
+
 /* reads a text file as input, converting each line
 into a wide character representation and applies several
 filters. */
@@ -545,6 +628,8 @@ void w_process_file(FILE *input,
   wchar_t *wcp;
   char wcq[MB_LEN_MAX+1];
 
+  set_iobuf_mode(input);
+
   /* initialize the norex state */
   tokbuf[0] = DIAMOND;
   tokbuf[1] = '\0';
