@@ -102,6 +102,7 @@ extern int optind, opterr, optopt;
 
 extern options_t u_options;
 extern options_t m_options;
+extern char *extn;
 
 extern char *progname;
 extern char *inputfile;
@@ -116,10 +117,7 @@ display_state disp; /* used interactively */
 
 extern long system_pagesize;
 
-extern void *in_iobuf;
-extern void *out_iobuf;
-
-int cmd = 0;
+extern int cmd;
 
 /***********************************************************
  * MISCELLANEOUS FUNCTIONS                                 *
@@ -213,8 +211,19 @@ void build_scores(weight_t *s) {
   lambda = (cat[0].model_num_docs > 0) ? 
     (cat[0].model_full_token_count / cat[0].model_num_docs) : 100;
 
+  if( m_options & (1<<M_OPTION_CALCENTROPY) ) {
+    cat[0].score_shannon = 
+      -( cat[0].score_shannon / ((weight_t)cat[0].complexity) -
+	 log((weight_t)cat[0].complexity) );
+    cat[0].score_div = 
+      -(cat[0].score/cat[0].complexity + cat[0].score_shannon);
+  }
+
+  /* various experimental scoring systems */
   s[0] = -cat[0].score - log_poisson(cat[0].complexity, lambda); /* arbitrary */
   s[1] = -cat[0].score / cat[0].complexity;
+  s[2] = cat[0].score_div;
+  s[3] = (weight_t)gamma_pvalue(&cat[0], cat[0].score_div)/10;
 }
 
 /* formats descriptions for email.index_format > 0 */
@@ -238,6 +247,7 @@ void build_other_descriptions(char **d) {
 /* fills the emails list one line at a time */
 char *process_email_line(char *textbuf) {
   regex_count_t r;
+
   if( mbox.prev_line_empty ) {
     if( strncmp(textbuf, "From ", 5) == 0 ) {
 
@@ -298,9 +308,8 @@ char *process_email_line(char *textbuf) {
 	  emails.list[emails.num_emails - 1].state |= (1<<STATE_LIMITED);
 	}
       }
-      emails.header.from[0] = '\0';
-      emails.header.subject[0] = '\0';
-
+      memset(emails.header.from, 0, HEADER_BUFLEN);
+      memset(emails.header.subject, 0, HEADER_BUFLEN);
     } 
   } 
 
@@ -773,7 +782,7 @@ void interactive_categorize() {
   if( (s = get_input_line("Sort current mailbox by category: ", "")) ) { 
 
     if( s[0] != '\0' ) {
-      cat[1].fullfilename = sanitize_path(s);
+      cat[1].fullfilename = sanitize_path(s, extn);
       if( load_category(&cat[1]) && 
 	  (input = fopen(emails.filename, "rb")) ) {
 
@@ -781,7 +790,7 @@ void interactive_categorize() {
 	ephemeral_message("Please wait, recalculating scores");
 	/* loaded category successfully, now free old resources */
 	free_category(&cat[0]);
-	memcpy(&cat[0], &cat[1], sizeof(Category));
+	memcpy(&cat[0], &cat[1], sizeof(category_t));
       
 	free_emails();
 	init_emails();
@@ -897,8 +906,8 @@ void redraw_current_state() {
   SLsmg_set_color(2);
 
   SLsmg_gotorc(0,0);
-  SLsmg_printf("mailinspect %20s sorted by category %s", 
-	       emails.filename, cat[0].filename);
+  SLsmg_printf("mailinspect %s %ld msgs sorted(%d) by category %s", 
+	       emails.filename, (long)emails.num_limited, emails.score_type, cat[0].filename);
   SLsmg_erase_eol();
 
   SLsmg_gotorc(r + 1, 0);
@@ -1387,8 +1396,6 @@ int main(int argc, char **argv) {
 
   /* setup */
 
-  m_options |= (1<<M_OPTION_XML); /* always use this with email */
-
   if( m_options & (1<<M_OPTION_CALCENTROPY) ) {
     init_empirical(&empirical, 
 		   default_max_tokens, 
